/*
 * Copyright 2019 Lightbend Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import * as protobufs from "../proto/protobuf-bundle";
import AnySupport from "./protobuf-any";

export interface CrdtData {
    /**
     * Get any deltas generated by modifying this CRDT, and reset the delta
     * to unchanged.
     */
    getAndResetDelta(): protobufs.cloudstate.crdt.CrdtDelta

    /**
     * Get the state of the CRDT, and reset the delta to unchanged.
     */
    getStateAndResetDelta(): protobufs.cloudstate.crdt.CrdtState

    /**
     * Apply the given delta to the CRDT.
     */
    applyDelta(delta: protobufs.cloudstate.crdt.CrdtDelta, anySupport: AnySupport,
               createCrdtFromState: (state: protobufs.cloudstate.crdt.CrdtState) => CrdtData): void

    /**
     * Apply the given state to the CRDT.
     */
    applyState(state: protobufs.cloudstate.crdt.CrdtState, anySupport: AnySupport,
               createCrdtFromState: (state: protobufs.cloudstate.crdt.CrdtState) => CrdtData): void
}

